#include "Listen.h"

#include "wrapp_cpp_function.inc"
#include "ConstDefine.h"
#include "../FLTService.h"
#include "../ThreadUtil.h"
#include "../json.hpp"
using json = nlohmann::json;

CB_S_I_S_S NewBaseCallBack(std::shared_ptr<flutter::MethodResult<flutter::EncodableValue>> result)
{

    auto t = std::make_shared<BaseCallBack>(result);
    auto f = [t](const std::string &operationID, int errCode, const std::string &errMsg, const std::string &data)
    {
        t->HandleCallback(operationID, errCode, errMsg, data);
    };

    return _wrapper_callonce_cpp_function(f);
}

///////////////////////////////////////////////////////////////////////////////////////////////

void InitListenCallBack::HandleCallback(int state, const std::string &msg)
{
    flutter::EncodableMap arguments;
    switch (ConstDefine(state))
    {
    case ConstDefine::CONNECTING:
        arguments.insert(std::make_pair("type", "onConnecting"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::CONNECT_SUCCESS:
        arguments.insert(std::make_pair("type", "onConnectSuccess"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::CONNECT_FAILED:
        arguments.insert(std::make_pair("type", "onConnectFailed"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::KICKED_OFFLINE:
        arguments.insert(std::make_pair("type", "onKickedOffline"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::USER_TOKEN_EXPIRED:
        arguments.insert(std::make_pair("type", "onUserTokenExpired"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::USER_TOKEN_INVALID:
        arguments.insert(std::make_pair("type", "onUserTokenExpired"));
        arguments.insert(std::make_pair("data", msg));
        break;
    }
    notifyEvent("connectListener", arguments);
}

CB_I_S NewInitListenCallBack()
{
    auto t = std::make_shared<InitListenCallBack>();
    auto f = [t](int state, const std::string &msg)
    {
        t->HandleCallback(state, msg);
    };

    return _wrapper_cpp_function(f);
}
///////////////////////////////////////////////////////////////////////////////////////////////

void UploadFileListenCallBack::HandleCallback(int state, const std::string &msg)
{
    flutter::EncodableMap arguments;
    flutter::EncodableMap data;
    json j;
    switch (ConstDefine(state))
    {
    case ConstDefine::OPEN:
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("size", std::stoi(msg)));
        arguments.insert(std::make_pair("type", "open"));
        arguments.insert(std::make_pair("data", data));
        break;
    case ConstDefine::PART_SIZE:
        // 需要使用json反序列化对象msg，使用临时结构
        j = json::parse(msg);
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("partSize", j.at("partSize").get<int>()));
        data.insert(std::make_pair("num", j.at("num").get<int>()));
        arguments.insert(std::make_pair("type", "partSize"));
        arguments.insert(std::make_pair("data", data));
        break;
    case ConstDefine::HASH_PART_PROGRESS:
        j = json::parse(msg);
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("index", j.at("index").get<int>()));
        data.insert(std::make_pair("size", j.at("size").get<int>()));
        data.insert(std::make_pair("partHash", j.at("partHash").get<std::string>()));
        arguments.insert(std::make_pair("type", "hashPartProgress"));
        arguments.insert(std::make_pair("data", data));
        break;
    case ConstDefine::HASH_PART_COMPLETE:
        j = json::parse(msg);
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("partHash", j.at("partHash").get<std::string>()));
        data.insert(std::make_pair("fileHash", j.at("fileHash").get<std::string>()));
        arguments.insert(std::make_pair("type", "hashPartComplete"));
        arguments.insert(std::make_pair("data", data));
        break;
    case ConstDefine::UPLOAD_ID:
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("uploadID", msg));
        arguments.insert(std::make_pair("type", "uploadID"));
        arguments.insert(std::make_pair("data", data));
        break;
    case ConstDefine::UPLOAD_PART_COMPLETE:
        j = json::parse(msg);
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("index", j.at("index").get<int>()));
        data.insert(std::make_pair("partSize", j.at("partSize").get<int>()));
        data.insert(std::make_pair("partHash", j.at("partHash").get<std::string>()));
        arguments.insert(std::make_pair("type", "uploadPartComplete"));
        arguments.insert(std::make_pair("data", data));
        break;
    case ConstDefine::UPLOAD_COMPLETE:
        j = json::parse(msg);
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("fileSize", j.at("fileSize").get<int>()));
        data.insert(std::make_pair("streamSize", j.at("streamSize").get<int>()));
        data.insert(std::make_pair("storageSize", j.at("storageSize").get<int>()));
        arguments.insert(std::make_pair("type", "uploadProgress"));
        arguments.insert(std::make_pair("data", data));
        break;
    case ConstDefine::COMPLETE:
        j = json::parse(msg);
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("size", j.at("fileSize").get<int>()));
        data.insert(std::make_pair("url", j.at("url").get<std::string>()));
        data.insert(std::make_pair("type", j.at("fileSize").get<int>()));
        arguments.insert(std::make_pair("type", "complete"));
        arguments.insert(std::make_pair("data", data));
        break;
    }
    notifyEvent("uploadFileListener", arguments);
}

CB_I_S NewUploadFileListenCallBack(std::string id)
{
    auto t = std::make_shared<UploadFileListenCallBack>(id);
    auto f = [t](int state, const std::string &msg)
    {
        t->HandleCallback(state, msg);
    };

    return _wrapper_cpp_function(f);
}

///////////////////////////////////////////////////////////////////////////////////////////////

void UploadLogsListenCallBack::HandleCallback(int state, const std::string &msg)
{
    flutter::EncodableMap arguments;
    flutter::EncodableMap data;
    json j;
    switch (ConstDefine(state))
    {
    case ConstDefine::ON_PROGRESS:
        j = json::parse(msg);
        data.insert(std::make_pair("current", j.at("current").get<int>()));
        data.insert(std::make_pair("size", j.at("size").get<int>()));
        arguments.insert(std::make_pair("type", "onProgress"));
        arguments.insert(std::make_pair("data", data));
        break;
    }
    notifyEvent("uploadLogsListener", arguments);
}

CB_I_S NewUploadLogsListenCallBack()
{
    auto t = std::make_shared<UploadLogsListenCallBack>();
    auto f = [t](int state, const std::string &msg)
    {
        t->HandleCallback(state, msg);
    };

    return _wrapper_cpp_function(f);
}
///////////////////////////////////////////////////////////////////////////////////////////////

void UserListenCallBack::HandleCallback(int state, const std::string &msg)
{
    flutter::EncodableMap arguments;
    flutter::EncodableMap data;
    json j;
    switch (ConstDefine(state))
    {
    case ConstDefine::SELF_INFO_UPDATED:

        arguments.insert(std::make_pair("type", "onSelfInfoUpdated"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::USER_STATUS_CHANGED:
        arguments.insert(std::make_pair("type", "onUserStatusChanged"));
        arguments.insert(std::make_pair("data", msg));
        break;
    }
    notifyEvent("userListener", arguments);
}

CB_I_S NewUserListenCallBack()
{
    auto t = std::make_shared<UserListenCallBack>();
    auto f = [t](int state, const std::string &msg)
    {
        t->HandleCallback(state, msg);
    };

    return _wrapper_cpp_function(f);
}

///////////////////////////////////////////////////////////////////////////////////////////////

void AdvancedMsgListenCallBack::HandleCallback(int state, const std::string &msg)
{
    flutter::EncodableMap arguments;
    flutter::EncodableMap data;
    json j;
    switch (ConstDefine(state))
    {
    case ConstDefine::MSG_DELETED:
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("message", msg));
        arguments.insert(std::make_pair("type", "onMsgDeleted"));
        arguments.insert(std::make_pair("data", data));
        break;
    case ConstDefine::NEW_RECV_MESSAGE_REVOKED:
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("messageRevoked", msg));
        arguments.insert(std::make_pair("type", "onNewRecvMessageRevoked"));
        arguments.insert(std::make_pair("data", data));
        break;
    case ConstDefine::NEW_RECV_MESSAGE_EDIT:
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("messageEdited", msg));
        arguments.insert(std::make_pair("type", "onNewRecvMessageEdited"));
        arguments.insert(std::make_pair("data", data));
        break;
    case ConstDefine::RECV_C2C_READ_RECEIPT:
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("msgReceiptList", msg));
        arguments.insert(std::make_pair("type", "onRecvC2CReadReceipt"));
        arguments.insert(std::make_pair("data", data));
        break;
    case ConstDefine::RECV_NEW_MESSAGE:
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("message", msg));
        arguments.insert(std::make_pair("type", "onRecvNewMessage"));
        arguments.insert(std::make_pair("data", data));
        break;
    case ConstDefine::RECV_OFFLINE_NEW_MESSAGE:
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("message", msg));
        arguments.insert(std::make_pair("type", "onRecvOfflineNewMessage"));
        arguments.insert(std::make_pair("data", data));
        break;
    case ConstDefine::RECV_ONLINE_ONLY_MESSAGE:
        data.insert(std::make_pair("id", id));
        data.insert(std::make_pair("message", msg));
        arguments.insert(std::make_pair("type", "onRecvOnlineOnlyMessage"));
        arguments.insert(std::make_pair("data", data));
        break;
    }
    notifyEvent("advancedMsgListener", arguments);
}

CB_I_S NewAdvancedMsgListenCallBack(std::string id)
{
    auto t = std::make_shared<AdvancedMsgListenCallBack>(id);
    auto f = [t](int state, const std::string &msg)
    {
        t->HandleCallback(state, msg);
    };

    return _wrapper_cpp_function(f);
}
///////////////////////////////////////////////////////////////////////////////////////////////

void MsgSendListenerCallBack::HandleCallback(const std::string &operationID, int state, const std::string &error, const std::string &msg, long progress)
{
    if (result == nullptr)
    {
        return;
    }

    if (state == 0 && progress == 0 && msg.empty())
    {
        flutter::EncodableMap arguments;
        flutter::EncodableMap data;
        json j;
        data.insert(std::make_pair("clientMsgID", clientMsgID));
        data.insert(std::make_pair("progress", progress));
        arguments.insert(std::make_pair("type", "onProgress"));
        arguments.insert(std::make_pair("data", data));
        notifyEvent("msgSendProgressListener", arguments);
    }
    else
    {
        // Create shared pointers for the data to ensure copy constructibility
        auto resultCopy = result;
        auto dataCopy = std::make_shared<std::string>(msg);
        auto errCodeCopy = state;
        auto errMsgCopy = std::make_shared<std::string>(error);

        // Ensure callback runs on the platform thread
        ThreadUtil::RunOnPlatformThread([resultCopy, errCodeCopy, errMsgCopy, dataCopy]()
                                        {
        if (errCodeCopy == 0) {
            resultCopy->Success(flutter::EncodableValue(*dataCopy));
        } else {
            resultCopy->Error(std::to_string(errCodeCopy), *errMsgCopy,
                            flutter::EncodableValue(*dataCopy));
        } });
    }
}

CB_S_I_S_S_I NewMsgSendCallBack(std::shared_ptr<flutter::MethodResult<flutter::EncodableValue>> result, std::string clientMsgID)
{
    auto t = std::make_shared<MsgSendListenerCallBack>(result, clientMsgID);
    auto f = [t](const std::string &operationID, int state, const std::string &error, const std::string &msg, long progress)
    {
        t->HandleCallback(operationID, state, error, msg, progress);
    };

    return _wrapper_callonce_cpp_function(f);
}

///////////////////////////////////////////////////////////////////////////////////////////////
void CustomBusinessListenCallBack::HandleCallback(int state, const std::string &msg)
{
    flutter::EncodableMap arguments;
    flutter::EncodableMap data;
    json j;
    switch (ConstDefine(state))
    {
    case ConstDefine::RECV_CUSTOM_BUSINESS_MESSAGE:

        arguments.insert(std::make_pair("type", "onRecvCustomBusinessMessage"));
        arguments.insert(std::make_pair("data", msg));
        break;
    }
    notifyEvent("customBusinessListener", arguments);
}

CB_I_S NewCustomBusinessListenCallBack()
{
    auto t = std::make_shared<CustomBusinessListenCallBack>();
    auto f = [t](int state, const std::string &msg)
    {
        t->HandleCallback(state, msg);
    };

    return _wrapper_callonce_cpp_function(f);
}

///////////////////////////////////////////////////////////////////////////////////////////////

void GroupListenerCallBack::HandleCallback(int state, const std::string &msg)
{
    flutter::EncodableMap arguments;
    flutter::EncodableMap data;

    switch (ConstDefine(state))
    {
    case ConstDefine::GROUP_APPLICATION_ACCEPTED:

        arguments.insert(std::make_pair("type", "onGroupApplicationAccepted"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::GROUP_APPLICATION_ADDED:

        arguments.insert(std::make_pair("type", "onGroupApplicationAdded"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::GROUP_APPLICATION_DELETED:

        arguments.insert(std::make_pair("type", "onGroupApplicationDeleted"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::GROUP_APPLICATION_REJECTED:

        arguments.insert(std::make_pair("type", "onGroupApplicationRejected"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::GROUP_DISMISSED:

        arguments.insert(std::make_pair("type", "onGroupDismissed"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::GROUP_INFO_CHANGED:

        arguments.insert(std::make_pair("type", "onGroupInfoChanged"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::GROUP_MEMBER_ADDED:

        arguments.insert(std::make_pair("type", "onGroupMemberAdded"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::GROUP_MEMBER_DELETED:

        arguments.insert(std::make_pair("type", "onGroupMemberDeleted"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::GROUP_MEMBER_INFO_CHANGED:

        arguments.insert(std::make_pair("type", "onGroupMemberInfoChanged"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::JOINED_GROUP_ADDED:

        arguments.insert(std::make_pair("type", "onJoinedGroupAdded"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::JOINED_GROUP_DELETED:

        arguments.insert(std::make_pair("type", "onJoinedGroupDeleted"));
        arguments.insert(std::make_pair("data", msg));
        break;
    }
    notifyEvent("groupListener", arguments);
}

CB_I_S NewGroupListenerCallBack()
{
    auto t = std::make_shared<GroupListenerCallBack>();
    auto f = [t](int state, const std::string &msg)
    {
        t->HandleCallback(state, msg);
    };

    return _wrapper_cpp_function(f);
}

///////////////////////////////////////////////////////////////////////////////////////////////

void ConversationListenerCallBack::HandleCallback(int state, const std::string &msg)
{
    flutter::EncodableMap arguments;
    flutter::EncodableMap data;
    json j;
    switch (ConstDefine(state))
    {
    case ConstDefine::CONVERSATION_CHANGED:

        arguments.insert(std::make_pair("type", "onConversationChanged"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::CONVERSATION_USER_INPUT_STATUS_CHANGED:

        arguments.insert(std::make_pair("type", "onConversationUserInputStatusChanged"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::NEW_CONVERSATION:

        arguments.insert(std::make_pair("type", "onNewConversation"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::SYNC_SERVER_FAILED:
        j = json::parse(msg);
        arguments.insert(std::make_pair("type", "onSyncServerFailed"));
        arguments.insert(std::make_pair("data", j.at("reinstalled").get<bool>()));
        break;
    case ConstDefine::SYNC_SERVER_FINISH:
        j = json::parse(msg);
        arguments.insert(std::make_pair("type", "onSyncServerFinish"));
        arguments.insert(std::make_pair("data", j.at("reinstalled").get<bool>()));
        break;
    case ConstDefine::SYNC_SERVER_START:
        j = json::parse(msg);
        arguments.insert(std::make_pair("type", "onSyncServerStart"));
        arguments.insert(std::make_pair("data", j.at("reinstalled").get<bool>()));
        break;
    case ConstDefine::SYNC_SERVER_PROGRESS:
        j = json::parse(msg);
        arguments.insert(std::make_pair("type", "onSyncServerProgress"));
        arguments.insert(std::make_pair("data", j.at("progress").get<int>()));
        break;
    case ConstDefine::TOTAL_UNREAD_MESSAGE_COUNT_CHANGED:
        j = json::parse(msg);
        arguments.insert(std::make_pair("type", "onTotalUnreadMessageCountChanged"));
        arguments.insert(std::make_pair("data", j.at("count").get<int>()));
        break;
    }
    notifyEvent("conversationListener", arguments);
}

CB_I_S NewConversationListenerCallBack()
{
    auto t = std::make_shared<ConversationListenerCallBack>();
    auto f = [t](int state, const std::string &msg)
    {
        t->HandleCallback(state, msg);
    };

    return _wrapper_cpp_function(f);
}
///////////////////////////////////////////////////////////////////////////////////////////////

void FriendshipListenCallBack::HandleCallback(int state, const std::string &msg)
{
    flutter::EncodableMap arguments;
    flutter::EncodableMap data;

    switch (ConstDefine(state))
    {
    case ConstDefine::BLACK_ADDED:

        arguments.insert(std::make_pair("type", "onBlackAdded"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::BLACK_DELETED:

        arguments.insert(std::make_pair("type", "onBlackDeleted"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::FRIEND_ADDED:

        arguments.insert(std::make_pair("type", "onFriendAdded"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::FRIEND_APPLICATION_ACCEPTED:

        arguments.insert(std::make_pair("type", "onFriendApplicationAccepted"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::FRIEND_APPLICATION_ADDED:

        arguments.insert(std::make_pair("type", "onFriendApplicationAdded"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::FRIEND_APPLICATION_DELETED:

        arguments.insert(std::make_pair("type", "onFriendApplicationDeleted"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::FRIEND_APPLICATION_REJECTED:

        arguments.insert(std::make_pair("type", "onFriendApplicationRejected"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::FRIEND_DELETED:

        arguments.insert(std::make_pair("type", "onFriendDeleted"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::FRIEND_INFO_CHANGED:

        arguments.insert(std::make_pair("type", "onFriendInfoChanged"));
        arguments.insert(std::make_pair("data", msg));
        break;
    }
    notifyEvent("friendListener", arguments);
}

CB_I_S NewFriendshipListenCallBack()
{
    auto t = std::make_shared<FriendshipListenCallBack>();
    auto f = [t](int state, const std::string &msg)
    {
        t->HandleCallback(state, msg);
    };

    return _wrapper_cpp_function(f);
}

///////////////////////////////////////////////////////////////////////////////////////////////

void ChannelListenCallBack::HandleCallback(int state, const std::string &msg)
{
    flutter::EncodableMap arguments;
    flutter::EncodableMap data;

    switch (ConstDefine(state))
    {
    case ConstDefine::CHANNEL_DISMISSED:

        arguments.insert(std::make_pair("type", "onChannelDismissed"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::CHANNEL_INFO_CHANGED:

        arguments.insert(std::make_pair("type", "onChannelInfoChanged"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::CHANNEL_MEMBER_ADDED:

        arguments.insert(std::make_pair("type", "onChannelMemberAdded"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::CHANNEL_MEMBER_DELETED:

        arguments.insert(std::make_pair("type", "onChannelMemberDeleted"));
        arguments.insert(std::make_pair("data", msg));
        break;
    case ConstDefine::CHANNEL_MEMBER_INFO_CHANGED:

        arguments.insert(std::make_pair("type", "onChannelMemberInfoChanged"));
        arguments.insert(std::make_pair("data", msg));
        break;
    }
    notifyEvent("channelListener", arguments);
}

CB_I_S NewChannelListenCallBack()
{
    auto t = std::make_shared<ChannelListenCallBack>();
    auto f = [t](int state, const std::string &msg)
    {
        t->HandleCallback(state, msg);
    };

    return _wrapper_cpp_function(f);
}
