#ifndef LISTEN_H
#define LISTEN_H

#include "flutter/encodable_value.h"
#include "flutter/method_result.h"
#include <libopenimsdk.h>

#include <functional>
#include <string>
#include <memory>
#include <iostream>
#include <bitset>
#include <atomic>
#include <thread>
#include <chrono>
#include <mutex>
#include "../ThreadUtil.h"

class BaseCallBack
{
public:
    BaseCallBack(std::shared_ptr<flutter::MethodResult<flutter::EncodableValue>> result_ptr)
        : result(result_ptr)
    {
    }

    std::shared_ptr<flutter::MethodResult<flutter::EncodableValue>> result = nullptr;

    void HandleCallback(const std::string &operationID, int errCode, const std::string &errMsg,
                        const std::string &data)
    {
        if (result == nullptr)
        {
            return;
        }

        // Create shared pointers for the data to ensure copy constructibility
        auto resultCopy = result;
        auto dataCopy = std::make_shared<std::string>(data);
        auto errCodeCopy = errCode;
        auto errMsgCopy = std::make_shared<std::string>(errMsg);

        // Ensure callback runs on the platform thread
        ThreadUtil::RunOnPlatformThread([resultCopy, errCodeCopy, errMsgCopy, dataCopy]()
                                        {
            if (errCodeCopy == 0) {
                resultCopy->Success(flutter::EncodableValue(*dataCopy));
            } else {
                resultCopy->Error(std::to_string(errCodeCopy), *errMsgCopy,
                                flutter::EncodableValue(*dataCopy));
            } });
    }
};

CB_S_I_S_S NewBaseCallBack(std::shared_ptr<flutter::MethodResult<flutter::EncodableValue>> result);

/////////////////////////////////////////////////////////////////////////////////////////////////
class InitListenCallBack
{
public:
    InitListenCallBack()
    {
    }

    void HandleCallback(int state, const std::string &msg);
};

CB_I_S NewInitListenCallBack();
/////////////////////////////////////////////////////////////////////////////////////////////////

class UploadFileListenCallBack
{
public:
    UploadFileListenCallBack(std::string id) : id(id)
    {
    }
    std::string id = "";

    void HandleCallback(int state, const std::string &msg);
};

CB_I_S NewUploadFileListenCallBack(std::string id);

/////////////////////////////////////////////////////////////////////////////////////////////////

class UploadLogsListenCallBack
{
public:
    UploadLogsListenCallBack()
    {
    }

    void HandleCallback(int state, const std::string &msg);
};

CB_I_S NewUploadLogsListenCallBack();

/////////////////////////////////////////////////////////////////////////////////////////////////

class UserListenCallBack
{
public:
    UserListenCallBack()
    {
    }

    void HandleCallback(int state, const std::string &msg);
};

CB_I_S NewUserListenCallBack();
/////////////////////////////////////////////////////////////////////////////////////////////////

class AdvancedMsgListenCallBack
{
public:
    AdvancedMsgListenCallBack(std::string id) : id(id)
    {
    }
    std::string id = "";

    void HandleCallback(int state, const std::string &msg);
};

CB_I_S NewAdvancedMsgListenCallBack(std::string id);

/////////////////////////////////////////////////////////////////////////////////////////////////
class MsgSendListenerCallBack
{
public:
    MsgSendListenerCallBack(std::shared_ptr<flutter::MethodResult<flutter::EncodableValue>> result_ptr, std::string clientMsgID) : result(result_ptr), clientMsgID(clientMsgID)
    {
    }
    std::string clientMsgID = "";
    std::shared_ptr<flutter::MethodResult<flutter::EncodableValue>> result = nullptr;
    void HandleCallback(const std::string &operationID, int state, const std::string &error, const std::string &msg, long progress);
};

CB_S_I_S_S_I NewMsgSendCallBack(std::shared_ptr<flutter::MethodResult<flutter::EncodableValue>> result, std::string clientMsgID);
/////////////////////////////////////////////////////////////////////////////////////////////////

class CustomBusinessListenCallBack
{
public:
    CustomBusinessListenCallBack()
    {
    }

    void HandleCallback(int state, const std::string &msg);
};
CB_I_S NewCustomBusinessListenCallBack();
/////////////////////////////////////////////////////////////////////////////////////////////////

class GroupListenerCallBack
{
public:
    GroupListenerCallBack()
    {
    }

    void HandleCallback(int state, const std::string &msg);
};

CB_I_S NewGroupListenerCallBack();

/////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////

class ConversationListenerCallBack
{
public:
    ConversationListenerCallBack()
    {
    }

    void HandleCallback(int state, const std::string &msg);
};

CB_I_S NewConversationListenerCallBack();
/////////////////////////////////////////////////////////////////////////////////////////////////

class FriendshipListenCallBack
{
public:
    FriendshipListenCallBack()
    {
    }

    void HandleCallback(int state, const std::string &msg);
};

CB_I_S NewFriendshipListenCallBack();
/////////////////////////////////////////////////////////////////////////////////////////////////
class ChannelListenCallBack {
    public:
        ChannelListenCallBack() {
        }
    
        void HandleCallback(int state, const std::string& msg);
    };
    
    CB_I_S NewChannelListenCallBack();
#endif